package com.akamai.player
{
	import com.akamai.hd.HDEvent;
	import com.akamai.hd.HDNetStream;
	import com.akamai.playeranalytics.AnalyticsPluginLoader;
	
	import flash.display.Loader;
	import flash.display.Sprite;
	import flash.display.StageDisplayState;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.KeyboardEvent;
	import flash.events.TimerEvent;
	import flash.media.Video;
	import flash.net.NetConnection;
	import flash.net.URLRequest;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.utils.Timer;

	public class HDVideoRenderer extends Sprite
	{
		
		private var _ns:HDNetStream;
		private var video:Video;
		private var componentWidth:Number;
		private var componentHeight:Number;
		private var slateLoader:Loader;
		private var lastPlayValue:String;
		private var retryTimer:Timer;
		private var waiting:Boolean;
		private var slatePath:String;
		private var tracePanel:TextField;
		
		public function HDVideoRenderer(width:Number,height:Number)
		{
			super();
			init(width,height);
		}
		
		public function set analyticsBeacon(value:String):void
		{
			_ns.analyticsBeacon = value;
		}
		
		public function get ns():HDNetStream
		{
			return _ns;
		}
		public function load(value:String,start:Number = NaN,end:Number = NaN):void
		{
			lastPlayValue = value;
			_ns.play(value,start,end - start);
		}
		public function set waitingSlate(value:String):void
		{
			slatePath = value;
		}
		public function setSize(width:Number,height:Number):void
		{
			componentWidth = width;
			componentHeight = height;
			tracePanel.width = width;
			tracePanel.height = height;
			scaleVideo(null);
			scaleSlate();
		}
		public function debug(value:String):void
		{
			onDebug(new HDEvent(HDEvent.DEBUG,value));
		}
		public function showWaitingSlate():void
		{
			onStreamNotFound(null);
		}
		private function scaleVideo(event:HDEvent):void
		{
			var videoWidth:Number = video.videoWidth == 0 ?  (event == null ? 640: event.data.width):video.videoWidth
			var videoHeight:Number = video.videoHeight == 0 ? (event == null ? 360: event.data.height):video.videoHeight
			if (this.stage == null)
			{
				video.smoothing = true;
			}
			else
			{
				video.smoothing  = (this.stage.displayState == StageDisplayState.NORMAL) && (videoWidth != componentWidth)
			}
			if (videoWidth/videoHeight > componentWidth/componentHeight)
			{
				video.width =  componentWidth;
				video.height =  componentWidth*videoHeight/videoWidth;
			}
			else
			{
				video.height =  componentHeight;
				video.width =  componentHeight*videoWidth/videoHeight;
			} 
			video.x = (componentWidth - video.width)/2;
			video.y = (componentHeight - video.height)/2;
		}
		private function scaleSlate():void
		{
			if (slateLoader.content != null)
			{
				if (slateLoader.content.width/slateLoader.content.height > componentWidth/componentHeight)
				{
					slateLoader.width =  componentWidth;
					slateLoader.height =  componentWidth*slateLoader.content.height /slateLoader.content.width;
				}
				else
				{
					slateLoader.height =  componentHeight;
					slateLoader.width =  componentHeight*slateLoader.content.width/slateLoader.content.height ;
				} 
				slateLoader.x = (componentWidth - slateLoader.width)/2;
				slateLoader.y = (componentHeight - slateLoader.height)/2;
			}
		}
		private function init(width:Number,height:Number):void
		{
			video = new Video(width,height);
			video.opaqueBackground = 0x2c2c2c;
			video.smoothing = true;
			video.deblocking = 1;
			addChild(video);
			var nc:NetConnection = new NetConnection();
			nc.connect(null);
			_ns = new HDNetStream(nc);
			_ns.startingIndex = 1;
			_ns.addEventListener(HDEvent.DEBUG,onDebug);
			_ns.addEventListener(HDEvent.STREAM_NOT_FOUND,onStreamNotFound);
			_ns.addEventListener(HDEvent.METADATA,onMetaData);
			_ns.addEventListener(HDEvent.SEEK,onSeek);
			video.attachNetStream(_ns);
			slateLoader = new Loader();
			slateLoader.visible = false;
			slateLoader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR,onSlateIOError);
			slateLoader.contentLoaderInfo.addEventListener(Event.COMPLETE,onSlateLoadComplete);
			addChild(slateLoader);
			tracePanel = new TextField();
			tracePanel.selectable = true;
			tracePanel.multiline = true;
			tracePanel.wordWrap = true;
			tracePanel.autoSize = "right";
			var tf:TextFormat = new TextFormat("Arial",10,0xFFFFFF,true);
			tracePanel.defaultTextFormat = tf;
			tracePanel.x = 0;
			tracePanel.visible = false;
			addChild(tracePanel);
			retryTimer = new Timer(60000,1);
			retryTimer.addEventListener(TimerEvent.TIMER_COMPLETE,onRetryTimerComplete);
			setSize(width,height);
			this.addEventListener(Event.ADDED_TO_STAGE,onAddedToStage);
			
		}
		private function onSeek(event:HDEvent):void
		{
			video.clear();
		}
		private function onAddedToStage(event:Event):void
		{
			this.stage.addEventListener(KeyboardEvent.KEY_DOWN, reportKeyDown);
		}
		private function reportKeyDown(event:KeyboardEvent):void
		{
    		if (event.charCode == 63)
    		{
    			tracePanel.visible = !tracePanel.visible;
    		}
    	}
		private function onSlateIOError(event:IOErrorEvent):void
		{
			// We don't do anything since we don't have a default slate to display
			trace(event.text);
		}
		private function onMetaData(event:HDEvent):void
		{
			slateLoader.visible = false;
			scaleVideo(event);
		}
		private function onRetryTimerComplete(event:TimerEvent):void
		{
			_ns.play(lastPlayValue);
		}
		private function onStreamNotFound(event:HDEvent):void
		{
			slateLoader.visible = true;
			if (slateLoader.content == null)
			{
				slateLoader.load(new URLRequest(slatePath));
			}
			retryTimer.reset();
			retryTimer.start();
		}
		
		private function onDebug(event:HDEvent):void
		{
			tracePanel.appendText((event.data as String)+"\n");
			if (tracePanel.text.length > 10000)
			{
				tracePanel.text = "Rolling trace log after 10,000 chars to preserve memory\n";
			}
			if (tracePanel.visible)
			{
				
				tracePanel.y = componentHeight - tracePanel.textHeight -10;
			}

		}
		private function onSlateLoadComplete(event:Event):void
		{
			scaleSlate();
		}
		
	}
}