package
{
	import com.akamai.hd.HDEvent;
	import com.akamai.hd.HDMBRObject;
	import com.akamai.hd.HDNetStream;
	import com.akamai.hd.HDSMILParser;
	import com.akamai.hdcore.samples.hdn1.TokenAuthenticationSMILPresenter;
	import com.akamai.hdcore.samples.utility.AkamaiMediaSamples;
	import com.akamai.hdcore.samples.utility.BasePresenter;
	import com.akamai.hdcore.samples.utility.BaseSample;
	import com.akamai.hdcore.samples.utility.MediaPlayerView;
	import com.akamai.logger.AkamaiConsoleLogger;
	import com.akamai.logger.AkamaiConsoleLoggerFactory;
	import com.akamai.logger.LoggerEvent;
	
	import flash.display.Sprite;
	import flash.events.ContextMenuEvent;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.NetStream;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.utils.describeType;
	
	import org.osmf.logging.Log;
	import org.osmf.logging.Logger;
	
	[SWF(width="640", height="390", frameRate="20", backgroundColor="#1a1a1a")]
	
	/**
	 * 
	 * This sample shows how to use HDN's Primary Token service with HDCore by 
	 * requesting a token for a SMIL file and setting the token on HDCore's 
	 * primary token property.
	 * 
	 * This sample also utilizes Akamai's player verification technology. Therefore, 
	 * it will not play the stream past (approximately) 10 -15 seconds and will abruptly stop.  
	 * Contact CCare or your Akamai representative to find out more about player verification 
	 * implementation in your player.
	 * 
	 */	
	public class TokenAuthenticationSMILSample extends BaseSample
	{
		
		private static const SECURE_SMIL_URL:String = AkamaiMediaSamples.SECURE_SMIL;
		
		private var mediaURL:String = SECURE_SMIL_URL;
		private var urlLoader:URLLoader;
		private var hostName:String;
		private var logger:AkamaiConsoleLogger;
		
		private var smilParser:HDSMILParser;
		
		public function TokenAuthenticationSMILSample()
		{
			addAkamaiConsoleLogger();
		}
		
		override protected function onAddedToStage(event:Event):void
		{
			super.onAddedToStage(event);
			MediaPlayerView(view).video.attachNetStream(netStream);
			HDNetStream(netStream).displayObject = this.stage;
		}
		
		override protected function getView():Sprite
		{
			return new MediaPlayerView();
		}
		
		override protected function getNetStream():NetStream
		{
			return new HDNetStream(netConnection);
		}
		
		override protected function getPresenter():BasePresenter
		{
			var presenter:BasePresenter = new TokenAuthenticationSMILPresenter(MediaPlayerView(view), HDNetStream(netStream), netConnection);
			toggleControls(false);
			presenter.load("");
			parseSMIL();
			return presenter;
		}
		
		private function getFormattedTokenServiceURL(url:String):String
		{
			logger.debug(AkamaiMediaSamples.TOKEN_SERVICE_URL + url + AkamaiMediaSamples.SECURE_USER_AND_PASSWARD)
			return AkamaiMediaSamples.TOKEN_SERVICE_URL + url + AkamaiMediaSamples.SECURE_USER_AND_PASSWARD;
		}

		
		private function parseSMIL():void
		{
			smilParser = new HDSMILParser()
			smilParser.load(mediaURL)
			smilParser.addEventListener(HDEvent.PARSED, onSmilParse)
			
			function onSmilParse(event:HDEvent):void
			{
				var mbrObject:HDMBRObject = ((event.data as Array)[0] as HDMBRObject);				
				var streamName:String = "/"+mbrObject.streams[0].name;				
				makeTokenRequest(new URLRequest(getFormattedTokenServiceURL(streamName)));
				smilParser.removeEventListener(HDEvent.PARSED, onSmilParse);
			}
		}
		
		private function makeTokenRequest(request:URLRequest):void
		{
			urlLoader = new URLLoader();
			urlLoader.addEventListener(Event.COMPLETE, onURLLoaderRequestComplete);
			urlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, onSecurityError);
			urlLoader.addEventListener(IOErrorEvent.IO_ERROR, onIoError);
			urlLoader.load(request);
			
			function onURLLoaderRequestComplete(event:Event):void
			{
				var tokenizedURL:String = urlLoader.data as String;
				HDNetStream(netStream).primaryToken = tokenizedURL.split("?primaryToken=")[1];
				toggleControls(true);
				logger.debug("Token req completed and ready for sample to be played --> "+tokenizedURL);
				
				var mbrObject:HDMBRObject = (smilParser.mbrObjects[0] as HDMBRObject);
				mbrObject.primaryToken = tokenizedURL.split("?primaryToken=")[1];
				presenter.playArgs = [mbrObject];
			}
			
			function onSecurityError(event:SecurityErrorEvent):void
			{
				logger.debug("securityErrorHandler: " + event.text);
			}
			
			function onIoError(event:IOErrorEvent):void
			{
				logger.debug("ioErrorHandler: " + event.text);
			}
		}

		private function toggleControls(value:Boolean):void
		{
			MediaPlayerView(view).controlBar.visible = value;
			MediaPlayerView(view).posterFrameButton.visible = value;			
		}
		
		private function addAkamaiConsoleLogger():void
		{
			// Note: the logging framework used here is provisional and the API may change in future releases.
			Log.loggerFactory = new AkamaiConsoleLoggerFactory();
			logger = Log.getLogger(describeType(this).@name) as AkamaiConsoleLogger
			logger.setCallback(onLogEvent);
			AkamaiConsoleLogger.enable();
			function onLogEvent(event:LoggerEvent):void
			{
				MediaPlayerView(view).console.log(event.msg);
			}
		}
		
		override protected function onToggleTrace(event:ContextMenuEvent):void
		{
			// Note: the logging framework used here is provisional and the API may change in future releases.
			enableTrace = !enableTrace;
			AkamaiConsoleLogger.enable(enableTrace);
		}
	}
}